@@ -15,6 +15,8 @@ from t_nextgen.exceptions import (
 )
 from t_desktop.config import IS_WINDOWS_OS
 
+from t_nextgen.utils.wait_if_high_usage import wait_if_high_usage
+
 if IS_WINDOWS_OS:
     from pywinauto.application import WindowSpecification
     from pywinauto.findwindows import ElementNotFoundError
@@ -168,31 +170,38 @@ class ImportFileWindow(NextGenWindow):
             raise NextGenControlLostException("Lost control of NextGen Application")
 
         while self.import_edi_modal_exist() and datetime.now() < end_time:
+            wait_if_high_usage()
             if current_try > max_tries:
                 raise Exception(
                     "Could not import current payment folder or lost NextGen reference and could not recover"
                 )
             try:
-                self.app = self.desktop_app.connect_to_logged_in_screen()
                 self._process_import_modal()
             except (ElementNotFoundError, TimeoutError):
+                time.sleep(10)
+            except _ctypes.COMError:
+                wait_if_high_usage()
+                self.logger.warning(
+                    "Lost control of NextGen Application while processing import modal.Waiting 10 seconds."
+                )
+                self.desktop_app.app = self.desktop_app.connect_to_logged_in_screen()
                 current_try += 1
                 time.sleep(10)
 
         if datetime.now() >= end_time:
             raise NextGenImportTimeoutException("Could not import current payment folder within 3600 seconds")
 
+    @capture_screenshot_if_exception()
+    @retry(exceptions=_ctypes.COMError, tries=3, delay=20, backoff=2)
     def _process_import_modal(self) -> None:
         """Handle Import Processing Dialog."""
-        modal = self.desktop_app.dialog.child_window(title="NextGen", control_type="Window")
+        modal = self.desktop_app.dialog.child_window(title="NextGen", control_type="Window", found_index=0)
         if self._modal_exists(modal):
             buttons_list = modal.wrapper_object().descendants(control_type="Button")
             if len(buttons_list) == 2 and buttons_list[0].texts()[0] == "OK" and buttons_list[1].texts()[0] == "Close":
                 with contextlib.suppress(_ctypes.COMError):
                     modal.child_window(title="OK", auto_id="2", control_type="Button").click()
 
-    @capture_screenshot_if_exception()
-    @retry(exceptions=_ctypes.COMError, tries=5, delay=20, backoff=2)
     def _modal_exists(self, modal: WindowSpecification) -> bool:
         """Check if modal exists.
 
@@ -201,7 +210,16 @@ class ImportFileWindow(NextGenWindow):
         Returns:
             bool: True if modal exists, False otherwise
         """
-        return modal.exists()
+        try:
+            return modal.exists()
+        except (_ctypes.COMError, RuntimeError):
+            wait_if_high_usage()
+            self.logger.warning(
+                "Lost control of NextGen Application while checking modal existence. Waiting 10 seconds."
+            )
+            time.sleep(10)
+            self.desktop_app.app = self.desktop_app.connect_to_logged_in_screen()
+            return modal.exists()
 
     def convert_edi_files_to_excel(self, excel_file_path: str) -> None:
         """Converts EDI files to Excel format by interacting with the application's excel export functionality.
@@ -244,7 +262,7 @@ class ImportFileWindow(NextGenWindow):
         try:
             with contextlib.suppress(_ctypes.COMError):
                 return self.desktop_app.dialog.child_window(auto_id="FileDialog", control_type="Custom").exists(
-                    timeout=3, retry_interval=1
+                    timeout=3, retry_interval=0.001
                 )
         except ElementNotFoundError:
             return False
@@ -321,6 +339,7 @@ class ImportFileWindow(NextGenWindow):
         ).child_window(auto_id="FileDialog", control_type="Custom")
         self.desktop_app.set_text(closest_window.child_window(control_type="Edit"), export_path)
 
+    @retry(exceptions=(_ctypes.COMError, RuntimeError), tries=5, delay=10, backoff=2)
     def import_edi_modal_exist(self) -> bool:
         """Look at a reference to a modal on the next screen to see if the import is finished.
 
@@ -329,12 +348,21 @@ class ImportFileWindow(NextGenWindow):
         a _ctypes.COMError error is raised, which falls into the exception and returns True to
         maintain waiting for the download.
         """
-        with contextlib.suppress((_ctypes.COMError, TimeoutError, RuntimeError)):
-            return not self.desktop_app.dialog.child_window(
+        try:
+            report_viewer = self.desktop_app.dialog.child_window(
                 title="ERA Import Posting - Enhanced Report Mode", auto_id="ReportViewer", control_type="Window"
-            ).exists(timeout=5, retry_interval=1)
-            # Return True if a timeouterror or RuntimeError happens to prevent break the import
-        return True
+            )
+            return not report_viewer.exists(timeout=5, retry_interval=1)
+        except (ElementNotFoundError, TimeoutError):
+            return False
+        except (_ctypes.COMError, RuntimeError):
+            wait_if_high_usage()
+            time.sleep(20)
+            self.desktop_app.app = self.desktop_app.connect_to_logged_in_screen()
+            report_viewer = self.desktop_app.dialog.child_window(
+                title="ERA Import Posting - Enhanced Report Mode", auto_id="ReportViewer", control_type="Window"
+            )
+            return not report_viewer.exists(timeout=5, retry_interval=0.001)
 
     def patient_has_duplicated_import(self) -> bool:
         """This method checks if the patient has duplicated import.
@@ -349,7 +377,8 @@ class ImportFileWindow(NextGenWindow):
                 auto_id="65535",
                 control_type="Text",
                 title="Some files/checks were identified as duplicate imports.  Continue?",
-            ).exists(timeout=3, retry_interval=1)
+                found_index=0,
+            ).exists(timeout=3, retry_interval=0.001)
         return False
 
     def _handle_duplicate_import(self) -> None:
@@ -380,38 +409,56 @@ class ImportFileWindow(NextGenWindow):
                 auto_id="65535",
                 control_type="Text",
                 title="Please select either ERA files or Patient Pay files but not both.",
-            ).exists(timeout=3, retry_interval=1)
+                found_index=0,
+            ).exists(timeout=3, retry_interval=0.001)
         return False
 
+    @retry(tries=4, delay=2)
     def click_columns_button(self) -> None:
         """Click on the Columns button in the Import Window."""
         self.logger.debug("Clicking on Columns button in Import Window.")
-        with contextlib.suppress(_ctypes.COMError):
-            self.enhanced_report_mode_window.child_window(title="Columns", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(title="Columns", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(auto_id="FilterText", control_type="Edit").wait(
+            "ready", timeout=2, retry_interval=0.001
+        )
 
+    @retry(tries=4, delay=2)
     def click_totals_button(self) -> None:
         """Click on the Totals button in the Import Window."""
         self.logger.debug("Clicking on Totals button in Import Window.")
-        with contextlib.suppress(_ctypes.COMError):
-            self.enhanced_report_mode_window.child_window(title="Totals", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(title="Totals", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(auto_id="Totals", control_type="Custom").wait(
+            "ready", timeout=2, retry_interval=0.001
+        )
 
+    @retry(tries=4, delay=2)
     def click_options_button(self) -> None:
         """Click on the Options button in the Import Window."""
         self.logger.debug("Clicking on Options button in Import Window.")
-        with contextlib.suppress(_ctypes.COMError):
-            self.enhanced_report_mode_window.child_window(title="Options", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(title="Options", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(title="Column Text Wrapping", control_type="CheckBox").wait(
+            "ready", timeout=2, retry_interval=0.001
+        )
 
+    @retry(tries=4, delay=2)
     def click_header_footer_button(self) -> None:
         """Click on the Header/Footer button in the Import Window."""
         self.logger.debug("Clicking on Header/Footer button in Import Window.")
-        with contextlib.suppress(_ctypes.COMError):
-            self.enhanced_report_mode_window.child_window(title="Header/Footer", control_type="Button").click_input()
+        header_button = self.enhanced_report_mode_window.child_window(title="Header/Footer", control_type="Button")
+        header_button.wait("ready", timeout=10, retry_interval=0.001)
+        header_button.click_input()
+        self.enhanced_report_mode_window.child_window(title="Header Options", control_type="Group").wait(
+            "ready", timeout=10, retry_interval=0.001
+        )
 
+    @retry(tries=4, delay=2)
     def click_sorting_button(self) -> None:
         """Click on the Sorting button in the Import Window."""
         self.logger.debug("Clicking on Sorting button in Import Window.")
-        with contextlib.suppress(_ctypes.COMError):
-            self.enhanced_report_mode_window.child_window(title="Sorting", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(title="Sorting", control_type="Button").click_input()
+        self.enhanced_report_mode_window.child_window(auto_id="Sorting", control_type="Custom").wait(
+            "exists", timeout=2
+        )
 
     def uncheck_all_group_by_checkboxes(self, max_iterations: int = 10) -> None:
         """Uncheck all checkboxes in the Group By Window.
@@ -457,7 +504,7 @@ class ImportFileWindow(NextGenWindow):
         check_box = data_item.descendants(control_type="CheckBox")[index]
         self.desktop_app.untoggle_checkbox(check_box)
 
-    def unselect_count_check_boxes(self, max_iterations: int = 10):
+    def unselect_count_check_boxes(self, max_iterations: int = 10) -> None:
         """Unselect the Count checkboxes in the Columns Window.
 
         Args:
@@ -488,7 +535,7 @@ class ImportFileWindow(NextGenWindow):
         )
         self.desktop_app.untoggle_checkbox(count_record_check_box)
 
-    def click_refresh_report_button(self):
+    def click_refresh_report_button(self) -> None:
         """Click on the Refresh Report button."""
         self.logger.debug("Clicking on the Refresh Report button.")
         custom = self.enhanced_report_mode_window.child_window(auto_id="Me", control_type="Custom")
@@ -514,7 +561,7 @@ class ImportFileWindow(NextGenWindow):
         for check_box in check_boxes:
             self.desktop_app.untoggle_checkbox(check_box)
 
-    def click_close_button_in_import_posting_report_window(self):
+    def click_close_button_in_import_posting_report_window(self) -> None:
         """Click on the Close button in the Import Posting Report Window."""
         self.logger.debug("Clicking on the Close button in the Import Posting Report Window.")
         with contextlib.suppress(_ctypes.COMError):
