@@ -18,7 +18,7 @@ class TransactionLedgerWindow(NextGenWindow):
     """TransactionLedger class with methods to interact with transaction ledger window."""
 
     @property
-    def window(self):
+    def window(self) -> WindowSpecification:
         """Return the BatchLedger window element."""
         return self.desktop_app.dialog.child_window(auto_id="BatchLedger", control_type="Window")
 
@@ -67,12 +67,12 @@ class TransactionLedgerWindow(NextGenWindow):
         """
         return self._get_amount("Adjustment", "3")
 
-    @retry_if_pywin_error(retries=3, delay=3)  # type: ignore[misc]
+    @retry_if_pywin_error(retries=3, delay=2)  # type: ignore[misc]
     def click_header_from_menu_and_wait_for_batch_maintanance(self) -> None:
         """Clicks the Header button."""
         self.logger.debug("Clicking on the Header button.")
         header_button = self.window.child_window(title="cmdHeader", control_type="Button")
-        if header_button.exists():
+        if header_button.exists(timeout=2, retry_interval=0.001):
             with contextlib.suppress(_ctypes.COMError):
                 self.desktop_app.mouse_click_element(header_button, "left")
         else:
@@ -127,19 +127,23 @@ class TransactionLedgerWindow(NextGenWindow):
             None
         """
         self.logger.debug(f"Clicking on the row at index: {index}")
-        if not row.is_visible():
-            pane.set_focus()
-            self.desktop_app.mouse_click_element(pane, "left")
-            self.desktop_app.click_down_n_times(index + offset)
+        pane.set_focus()
+        page_size = 50 if self.window.is_maximized() else 20
+        self.desktop_app.click_center_and_scroll(self.window, index + offset, page_size=page_size)
         row.double_click_input()
-        modal = self.window.child_window(
-            title="Security has been specified for this batch.  This batch of "
-            "transactions is only accessible by its specified owner."
-        )
-        if modal.exists(timeout=1, retry_interval=1):
-            raise LockedBatchError("Batch locked: Security has been specified for this batch!")
-        else:
-            self.desktop_app.dialog.child_window(title="lstPayLedger", control_type="Pane").wait("ready", 10)
+
+        try:
+            self.desktop_app.dialog.child_window(auto_id="PaymentEntry", control_type="Window").wait(
+                "visible", 5, retry_interval=0.001
+            )
+        except TimeoutError as e:
+            modal = self.window.child_window(
+                title="Security has been specified for this batch.  This batch of "
+                "transactions is only accessible by its specified owner."
+            )
+            if modal.exists(timeout=1, retry_interval=0.001):
+                raise LockedBatchError("Batch locked: Security has been specified for this batch!")
+            raise e
 
     def get_row_elements(self) -> list[ListItemWrapper]:
         """Get all row elements.
@@ -179,7 +183,7 @@ class TransactionLedgerWindow(NextGenWindow):
                 row_source = row.children(title="Source")[0].get_value()
                 row_billed = convert_to_decimal(row.children(title="Billed")[0].get_value())
                 row_tran_code = row.children(title="Tran Code")[0].get_value()
-            except Exception as e:
+            except (IndexError, AttributeError, ValueError) as e:
                 self.logger.warning(f"Skipping row due to error: {e}")
                 continue
             source_match = row_source == source_value
